# coding: utf-8
import traceback
from celery_balancer import logger
from celery_balancer.balancer import Balancer
from celery_balancer.introspection import Introspector
from celery_balancer.models import Queue, Server, ServerWorkerM2M


class BalancerApi(object):

    @staticmethod
    def start_balancer(error_log_file='', info_log_file='', logs_path=''):
        """
        Запуск балансировщика.
        :param error_log_file: имя фала логов ошибок.
        :param info_log_file: имя файла логов информации.
        :param logs_path: папка с логами.
        """
        balancer = Balancer(error_log_file, info_log_file, logs_path)
        print u'Balancer is running now. \nPress "CTRL+C" to stop.'
        logger.info(u'Balancer started.')
        try:
            balancer.run()
        except KeyboardInterrupt:
            print (u'\nBalancer began to stop. \n'
                   u'Wait while console returns the control.')
            logger.info(u'Balancer began to stop.')
        except Exception:
            logger.error(traceback.format_exc())
            raise
        finally:
            balancer.stop_all_decisions_executions()

    @staticmethod
    def bind_workers_to_queues(error_log_file='',
                               info_log_file='', logs_path=''):
        """
        Привязка воркеров к их очередям.
        Эта процедура нужна, потому что при аварийной остановке балансировщика
        связь между воркером и очередью может быть потеряна.
        Запускается автоматически при запуске балансировщика.
        :param error_log_file: имя фала логов ошибок.
        :param info_log_file: имя файла логов информации.
        :param logs_path: папка с логами.
        :return: Success/traceback
        """
        try:
            balancer = Balancer(error_log_file, info_log_file, logs_path)
            balancer.bind_workers_to_queues()
        except Exception:
            logger.error(traceback.format_exc())
            raise
        else:
            print u'Success'

    @staticmethod
    def get_balancer_interval():
        """Получение периода балансировки в секундах."""
        return Introspector.get_balancer_interval()

    @staticmethod
    def get_queues_names():
        """
        Получение имен всех очередей.
        :return: кортеж имен очередей
        """
        return Introspector.get_queues_names()

    @staticmethod
    def get_servers_names():
        """
        Получение имен всех зарегистрированных серверов.
        :return: кортеж имен серверов
        """
        return Introspector.get_servers_names()

    @staticmethod
    def get_queue_settings_names():
        """
        Получение всех имен настроек очереди.
        :return: {имя настройки: комментарий, ...}
        """
        return Introspector.get_queue_settings_names()

    @staticmethod
    def get_server_settings_names():
        """
        Получение всех имен настроек сервера.
        :return: {имя настройки: комментарий, ...}
        """
        return Introspector.get_server_settings_names()

    @staticmethod
    def get_queue_states_names():
        """
        Получение всех имен состояний очереди.
        :return: {имя состояния: комментарий, ...}
        """
        return Introspector.get_queue_states_names()

    @staticmethod
    def get_server_metrics_names():
        """
        Получение всех имен метрик загруженности сервера.
        :return: {имя метрики: комментарий, ...}
        """
        return Introspector.get_server_metrics_names()

    @staticmethod
    def set_queues_settings(settings, queue_names_list=None):
        """
        Задание настроек очередей.
        :param settings: {имя настройки: значение настройки, ...}
        :param queue_names_list: список имен очередей. Еслип не указан,
        то настройка установится для всех очередей.
        :return: Success/traceback
        """
        logger.init_logging()
        try:
            Queue.objects.set_fields_values(settings, queue_names_list)
        except Exception:
            logger.error(traceback.format_exc())
            raise
        else:
            logger.info(u'set_queues_settings. '
                        u'\n Settings: %s. \n Queues: %s' %
                        (unicode(settings), unicode(queue_names_list)))
            print u'Success'

    @staticmethod
    def set_servers_settings(settings, server_names_list=None):
        """
        Задание настроек серверов.
        :param settings: {имя настройки: значение настройки, ..}
        :param server_names_list: список имен серверов. Еслип не указан,
        то настройка установится для всех серверов.
        :return: Success/traceback
        """
        logger.init_logging()
        try:
            Server.objects.set_fields_values(settings, server_names_list)
        except Exception:
            logger.error(traceback.format_exc())
            raise
        else:
            logger.info(u'set_servers_settings. '
                        u'\n Settings: %s. \n Servers: %s' %
                        (unicode(settings), unicode(server_names_list)))
            print u'Success'

    @staticmethod
    def get_queues_settings(settings_name_list=None, queue_name_list=None):
        """
        Получение настроек очереди.
        :param settings_name_list: список имен настроек. Если не указан,
        то будут получены все настройки.
        :param queue_name_list: список имен очередей. Еслип не указан,
        то настройки будут получены для всех очередей.
        :return: {имя очереди: {имя настройки: значение настройки, ...}, ...}
        """
        return Introspector.get_queues_settings(settings_name_list,
                                                queue_name_list)

    @staticmethod
    def get_servers_settings(settings_name_list=None, server_name_list=None):
        """
        Получение настроек сервера.
        :param settings_name_list: список имен настроек. Если не указан,
        то будут получены все настройки.
        :param server_name_list: список имен серверов. Еслип не указан,
        то настройки будут получены для всех серверов.
        :return: {имя сервера: {имя настройки: значение настройки, ...}, ...}
        """
        return Introspector.get_servers_settings(settings_name_list,
                                                 server_name_list)
    
    @staticmethod
    def get_queues_states(states_names_list=None, queues_names_list=None):
        """
        Получение состояний очереди.
        :param states_names_list: список имен состояний. Если не указан,
        то будут получены все состояния.
        :param queues_names_list: список имен очередей. Еслип не указан,
        то состояния будут получены для всех очередей.
        :return: {
            имя очереди: {
                имя состояния: значение состояния,
                ...,
                имя состояния: {имя воркера: значение состояния, ...},
                ...
            },
            ...
        }
        """
        return Introspector.get_queues_states(states_names_list,
                                              queues_names_list)

    @staticmethod
    def get_servers_metrics(metrics_names_list=None, servers_names_list=None):
        """
        Получение метрик загруженности сервера.
        :param metrics_names_list: список имен метрик. Если не указан,
        то будут получены все метрики.
        :param servers_names_list: список имен серверов. Еслип не указан,
        то метрики будут получены для всех серверов.
        :return: {имя сервера: {имя метрики: значение метрики, ...}, ...}
        """
        return Introspector.get_servers_metrics(metrics_names_list,
                                                servers_names_list)

    @staticmethod
    def register_server(name, host, username, password):
        """
        Регистрация сервера.
        Если сервер зарегистрирован, то по нему будет собираться информация
        о загруженности.
        :param name: имя сервера
        :param host: адрес сервера относительно балансировщика
        :param username: пользователь
        :param password: пароль
        :return: Success/traceback
        """
        logger.init_logging()
        try:
            Server.objects.register_server(name, host, username, password)
        except Exception:
            logger.error(traceback.format_exc())
            raise
        else:
            logger.info(u'register_server: name=%s, host=%s, username=%s' %
                        (name, host, username))
            print u'Success'

    @staticmethod
    def cancel_servers_registration(server_name_list):
        """
        Удаление регистрации сервера.
        :param server_name_list: список имен серверов.
        :return: Success/traceback
        """
        logger.init_logging()
        try:
            Server.objects.cancel_servers_registration(server_name_list)
        except Exception:
            logger.error(traceback.format_exc())
            raise
        else:
            logger.info(u'cancel_servers_registration. Servers names: %s' %
                        unicode(server_name_list))
            print u'Success'

    @staticmethod
    def get_queues_workers_names(queue_names_list=None):
        """
        Получение имен воркеров очередей.
        :param queue_names_list: список имен очередей. Еслип не указан,
        то имена воркеров будут получены для всех очередей.
        :return: {имя очереди: (список имен воркеров), ...}
        """
        return Introspector.get_queues_workers_names(queue_names_list)

    @staticmethod
    def set_servers_affected_by_worker(affection_dict):
        """
        Задание на загруженность каких серверов какие воркеры влияют.
        :param affection_dict: {имя воркера: (список имен серверов), ...}
        :return: Success/traceback
        """
        logger.init_logging()
        try:
            ServerWorkerM2M.objects.set_servers_affected_by_worker(
                affection_dict
            )
        except Exception:
            logger.error(traceback.format_exc())
            raise
        else:
            logger.info(u'set_servers_affected_by_worker.'
                        u'\n Affection dict: %s.' % unicode(affection_dict))
            print u'Success'

    @staticmethod
    def get_workers_names_affect_on_servers(server_name_list=None):
        """
        Получение имен воркеров, влияющих на загруженность серверов.
        :param server_name_list: список имен серверов. Если не указан,
        то будет получена информация по всем серверам.
        :return: {имя сервера: (список имен воркеров), ...}
        """
        return Introspector.get_workers_names_affect_on_servers(
            server_name_list
        )

    @staticmethod
    def refresh_balancer_tables():
        """
        Обновление таблиц балансировщика.
        Источник данных - settings.CELERY_QUEUES.
        Используется при настройке балансировщика.
        :return: Success/traceback
        """
        logger.init_logging()
        try:
            balancer = Balancer()
            balancer.refresh_balancer_tables()
        except Exception:
            logger.error(traceback.format_exc())
            raise
        else:
            print u'Success'