# coding: utf-8
from celery_balancer.balancer import Balancer
from celery_balancer.models import Queue, Server, Worker


class Introspector(object):
    """
    Класс, отвечающий за получение информации
    о балансировщике для пользователей api.
    """

    # Список имен состояний очередей c комментариями.
    QUEUE_STATES = {
        u'queued_tasks_cnt': u'Количество заданий в очереди',
        u'total_proc_cnt': u'Количество обработчиков воркера очереди.',
        u'active_proc_cnt': (u'Количество обработчиков воркера очереди, '
                             u'занятых работой.'),
        u'last_on_off_datetime': (u'Дата и время последнего '
                                  u'включения/выключения воркера очереди.'),
        u'last_decision_datetime': (
            u'Дата и время последнего решения об '
            u'изменении кол-ва обработчиков воркера очереди.'
        ),
        u'expected_proc_cnt': u'Ожидаемое число обработчиков',
    }

    # Состояния очередей, которые необходимо получать из воркеров.
    QUEUE_STATES_FROM_WORKERS_STATES = (
        u'total_proc_cnt',
        u'active_proc_cnt',
        u'last_on_off_datetime',
        u'last_decision_datetime',
        u'expected_proc_cnt',
    )

    @staticmethod
    def get_balancer_interval():
        balancer = Balancer()
        return balancer.get_balancer_interval()

    @staticmethod
    def get_queues_names():
        return Queue.objects.get_names()

    @staticmethod
    def get_servers_names():
        return Server.objects.get_names()

    @staticmethod
    def get_queue_settings_names():
        return Queue.objects.get_settings_names()

    @staticmethod
    def get_server_settings_names():
        return Server.objects.get_settings_names()

    @staticmethod
    def get_queues_settings(settings_name_list=None, queue_name_list=None):
        return Queue.objects.get_fields_values(settings_name_list,
                                               queue_name_list)

    @staticmethod
    def get_servers_settings(settings_name_list=None, server_name_list=None):
        settings = Server.objects.get_fields_values(settings_name_list,
                                                    server_name_list)
        for server_name, server_settings in settings.items():
            if 'password' in server_settings:
                # Пароль нельзя показывать
                server_settings['password'] = u'****'
        return settings

    @staticmethod
    def get_queues_workers_names(queue_names_list=None):
        return Queue.objects.get_queues_workers_names(queue_names_list)

    @staticmethod
    def get_workers_names_affect_on_servers(server_name_list=None):
        return Server.objects.get_workers_names_affect_on_servers(
            server_name_list
        )

    @staticmethod
    def get_server_metrics_names():
        balancer = Balancer()
        return balancer.metrics_api.metrics

    @staticmethod
    def get_servers_metrics(metrics_names_list=None, servers_names_list=None):
        balancer = Balancer()
        if not servers_names_list:
            servers_names_list = Server.objects.get_names()
        if not metrics_names_list:
            metrics_names_list = balancer.metrics_api.metrics.keys()
        res = dict()
        for server_name in servers_names_list:
            res[server_name] = dict()
            for metric in metrics_names_list:
                res[server_name][metric] = (
                    balancer.metrics_api.get_metric(metric, server_name)
                )

        return res

    @staticmethod
    def get_queue_states_names():
        return Introspector.QUEUE_STATES

    @staticmethod
    def get_queues_states(states_names_list=None, queues_names_list=None):
        if not states_names_list:
            states_names_list = Introspector.get_queue_states_names().keys()
        if not queues_names_list:
            queues_names_list = Introspector.get_queues_names()
        res = dict()
        for queue_name in queues_names_list:
            queue_state = dict()
            for state_name in states_names_list:
                if state_name in Introspector.QUEUE_STATES_FROM_WORKERS_STATES:
                    queue_state[state_name] = dict()
                    workers_name_list = Introspector.get_queues_workers_names(
                        (queue_name, )
                    )[queue_name]
                    for workers_name in workers_name_list:
                        queue_state[state_name][workers_name] = getattr(
                            Introspector, u''.join([u'_get_', state_name])
                        )(workers_name)
                else:
                    queue_state[state_name] = getattr(
                        Introspector, u''.join([u'_get_', state_name])
                    )(queue_name)
            res[queue_name] = queue_state
        return res

    @staticmethod
    def _get_queued_tasks_cnt(queue_name):
        balancer = Balancer()
        return balancer.mq_api.get_tasks_cnt(queue_name)

    @staticmethod
    def _get_total_proc_cnt(worker_name):
        balancer = Balancer()
        return balancer.celery_api.get_total_proc_cnt().get(worker_name, 0)

    @staticmethod
    def _get_active_proc_cnt(worker_name):
        balancer = Balancer()
        return balancer.celery_api.get_active_proc_cnt().get(worker_name, 0)

    @staticmethod
    def _get_last_on_off_datetime(worker_name):
        return Worker.objects.get_fields_values(
            (u'last_on_off_datetime', ), (worker_name, )
        )[worker_name][u'last_on_off_datetime']

    @staticmethod
    def _get_last_decision_datetime(worker_name):
        return Worker.objects.get_fields_values(
            (u'last_decision_datetime', ), (worker_name, )
        )[worker_name][u'last_decision_datetime']

    @staticmethod
    def _get_expected_proc_cnt(worker_name):
        return Worker.objects.get_fields_values(
            (u'expected_proc_cnt', ), (worker_name, )
        )[worker_name][u'expected_proc_cnt']