# coding: utf-8
import logging
import os
from django.conf import settings


ERROR_LOGGER_NAME = 'celery_balancer_error_logger'
INFO_LOGGER_NAME = 'celery_balancer_info_logger'


def init_logging(error_log_file='', info_log_file='', logs_path=''):

    if hasattr(logging, "celery_balancer_set_up_done"):
        return

    error_log_file = error_log_file or 'celery_balancer_error.log'
    info_log_file = info_log_file or 'celery_balancer_info.log'
    logs_path = logs_path or settings.LOG_PATH

    if not os.path.exists(logs_path):
        os.mkdir(logs_path)

    loggers = [
        (ERROR_LOGGER_NAME, logging.ERROR, error_log_file),
        (INFO_LOGGER_NAME, logging.INFO, info_log_file),
    ]

    formatter = logging.Formatter("[%(asctime)s] %(message)s")
    formatter.datefmt = '%Y-%m-%d %H:%M:%S'
    for lname, level, fname in loggers:
        t = logging.getLogger(lname)
        t.setLevel(level)
        handler = logging.FileHandler(os.path.join(logs_path, fname),
                                      encoding='utf-8')
        handler.setFormatter(formatter)
        t.addHandler(handler)

    logging.celery_balancer_set_up_done = True


def info(msg):
    logger = logging.getLogger(INFO_LOGGER_NAME)
    logger.info(msg)


def error(msg):
    logger = logging.getLogger(ERROR_LOGGER_NAME)
    logger.error(msg)
