# coding: utf-8
import abc
import traceback
from celery_balancer import logger


class MetricsApi(object):
    """
    Интерфейс для API получения метрик.

    Метрики перечисляем в metrics.

    Для добавления новой метрики нужно:
        1. Добавить ее {имя_метрики: комментарий} в metrics.
        2. Завести поле в модели Server с максимально допустимым значением
           метрики с именем max_имя_метрики.
        3. Реализовать метод получения метрики _get_имя_метрики, принимающий в
           себя имя сервера для которого требуется получить метрику

    Если нужно, чтобы метрика не участвовала в работе балансировщика,
    то либо возвращать None в методе _get_имя_метрики,
    либо выставить значение None в таблице Server для поля max_имя_метрики.
    """
    __metaclass__ = abc.ABCMeta

    @staticmethod
    @abc.abstractproperty
    def metrics():
        """
        :rtype : dict
        :return: {имя_метрики: комментарий}
        """
        pass

    def get_metric(self, metric_name, server_name):
        try:
            metric = getattr(
                self, u''.join([u'_get_', metric_name])
            )(server_name)
        except Exception:
            logger.error(traceback.format_exc())
            return None
        else:
            return metric