# coding: utf-8
import base64
from django.db import models
from django.db.models import CASCADE
from celery_balancer.managers import QueueManager, ServerManager, \
    WorkerManager, ServerWorkerM2MManager


# Ограничиваем длину имени, потому что по именам везде unique=True
NAME_MAX_LENGTH = 256


class Queue(models.Model):
    class Meta:
        db_table = 'celery_balancer_queue'
        #                |-----max_length_for_verbose_name-----|
        verbose_name = u'Очередь.'

    objects = QueueManager()

    name = models.CharField(u'Имя очереди', unique=True, max_length=NAME_MAX_LENGTH)
    priority = models.PositiveIntegerField(u'Приоритет', default=1)
    is_active = models.BooleanField(u'Активность очереди', default=False)
    decision_interval = models.IntegerField(u'Интервал в секундах, с которым будет приниматься решение по очереди', default=60)

    settings_fields_with_comments = {
        u'priority': u'1 - наивысший приоритет.',
        u'is_active': u'',
        u'decision_interval': u'',
    }

    def get_workers_names(self):
        return tuple(self.worker_set.values_list('name', flat=True))


class Worker(models.Model):
    class Meta:
        db_table = 'celery_balancer_worker'
        #                |-----max_length_for_verbose_name-----|
        verbose_name = u'Воркер.'

    objects = WorkerManager()

    queue = models.ForeignKey(Queue, verbose_name=u'Ссылка на очередь', on_delete=CASCADE)
    name = models.CharField(u'Имя воркера', unique=True, max_length=NAME_MAX_LENGTH)
    last_on_off_datetime = models.DateTimeField(u'Дата и время последнего включения/выключения', null=True)
    last_decision_datetime = models.DateTimeField(u'Дата и время последнего решения', null=True)
    expected_proc_cnt = models.PositiveIntegerField(u'Ожидаемое число обработчиков', null=True)


class Server(models.Model):
    class Meta:
        db_table = 'celery_balancer_server'
        #                |-----max_length_for_verbose_name-----|
        verbose_name = u'Сервер.'

    objects = ServerManager()

    name = models.CharField(u'Имя сервера', unique=True, max_length=NAME_MAX_LENGTH)
    host = models.CharField(u'Адрес сервера', max_length=32)
    username = models.CharField(u'Имя пользователя', max_length=32)
    # Пароль храним в шифрованном виде.
    password = models.CharField(u'Пароль', max_length=256)
    workers = models.ManyToManyField(Worker, verbose_name=u'Воркеры, влияющие на загруженность сервера', through='ServerWorkerM2M', null=True)

    # Метрики для определения загруженности сервера.
    max_mem = models.PositiveIntegerField(u'Максимальное допустимое использование оперативной памяти (в мегабайтах)', null=True)
    max_swp = models.PositiveIntegerField(u'Максимальное допустимое использование свопа (в мегабайтах)', null=True)
    max_load_avg_1 = models.PositiveIntegerField(u'Максимальное допустимое значение load average за 1 минуту', null=True)
    max_load_avg_5 = models.PositiveIntegerField(u'Максимальное допустимое значение load average за 5 минут', null=True)
    max_load_avg_15 = models.PositiveIntegerField(u'Максимальное допустимое значение load average за 15 минут', null=True)

    settings_fields_with_comments = {
        u'host': u'',
        u'username': u'',
        u'password': u'',
        u'max_mem': u'',
        u'max_swp': u'',
        u'max_load_avg_1': u'',
        u'max_load_avg_5': u'',
        u'max_load_avg_15': u'',
    }

    encryption_key = u'0000ede6-cf07-11e4-8af1-10604bacc988'

    @classmethod
    def encrypt(cls, plaintext):
        from simplecrypt import encrypt
        encryption_key = cls.encryption_key
        ciphertext = encrypt(encryption_key, plaintext)
        ciphertext = base64.encodestring(ciphertext)
        return ciphertext

    @classmethod
    def decrypt(cls, ciphertext):
        from simplecrypt import decrypt
        encryption_key = cls.encryption_key
        ciphertext = base64.decodestring(ciphertext)
        plaintext = decrypt(encryption_key, ciphertext)
        return plaintext


class ServerWorkerM2M(models.Model):
    class Meta:
        db_table = 'celery_balancer_server_worker_m2m'
        #                |-----max_length_for_verbose_name-----|
        verbose_name = u'Связь воркера и сервера.'

    objects = ServerWorkerM2MManager()

    server = models.ForeignKey(Server, verbose_name=u'Ссылка на сервер', on_delete=CASCADE)
    worker = models.ForeignKey(Worker, verbose_name=u'Ссылка на воркер', on_delete=CASCADE)
