# coding: utf-8
from django.conf import settings
from celery_balancer.models import Queue
from celery_balancer.mq_api.interface import MQApi


class RabbitMQApi(MQApi):
    """API очереди сообщений RabbitMQ"""

    def __init__(self, celery_api):
        self.celery_api = celery_api
        import pika
        params = pika.ConnectionParameters(
            host=settings.BROKER_HOST,
            port=int(settings.BROKER_PORT),
            virtual_host=settings.BROKER_VHOST,
            credentials=pika.credentials.PlainCredentials(
                settings.BROKER_USER, settings.BROKER_PASSWORD
            )
        )
        connection = pika.BlockingConnection(params)
        self.channel = connection.channel()

    def get_tasks_cnt(self, queue_name):
        """
        Получение кол-ва заданий в очереди.

        При использовании RabbitMQ в качестве очереди сообщений, есть нюанс.
        Когда воркер включен, то часть заданий он уже забрал,
        но еще не начал выполнять.
        Эти задания уже не в очереди, воркер их забрал,
        но они еще не выполняются. А с точки зрения балансировщика,
        они еще в очереди, так как не выполняются.
        В админке раббита они отображаются как "Unacked".
        API RabbitMQ дает неверные данные для этой колонки, так что приходится
        получать эти данные через CeleryApi (они называются "reserved"),
        что не очень изящно, но по-другому пока не удалось решить.
        :param queue_name: имя очереди
        """
        ready_tasks_cnt = self.channel.queue_declare(
            queue=queue_name,
            passive=True
        ).method.message_count

        unacked_tasks_cnt = 0
        all_workers_reserved_tasks_cnt = (
            self.celery_api.get_reserved_tasks_cnt()
        )
        queue_workers_names = Queue.objects.get_queues_workers_names(
            (queue_name, )
        )[queue_name]
        for worker_name in queue_workers_names:
            unacked_tasks_cnt += (
                all_workers_reserved_tasks_cnt.get(worker_name, 0)
            )

        return ready_tasks_cnt + unacked_tasks_cnt